/* eslint-disable arrow-body-style */
const axios = require('axios');
const https = require('https');

// *****************************************************
// SimRequester ************************************
// *****************************************************

const asToken = '6192289bbfbdb878bfc01b816d4297ae7ac2b6e3ad28b3ffcd4476c320130dc6';

const simRequestHelper = axios.create({
  httpsAgent: new https.Agent({
    rejectUnauthorized: false,
  }),
  baseURL: `http://localhost:8008/_matrix/client/r0`,
});

simRequestHelper.defaults.headers.post['Content-Type'] = 'application/json';
simRequestHelper.defaults.headers.post.Accept = '*/*';

// Add a response interceptor
simRequestHelper.interceptors.response.use((response) => {
  // Do something with response data
  return response.data;
}, (error) => {
  // Do something with response error
  return Promise.reject(error);
});

// *****************************************************
// wUC Api *********************************************
// *****************************************************

class SimApiRequester {
  static getJoinedRooms(token) {
    return simRequestHelper.get('/joined_rooms', {
      params: {
        access_token: token,
      },
    });
  }
  
  static joinRoom(token, roomId) {
    return simRequestHelper.post(`/rooms/${roomId}/join`, {}, {
      params: {
        access_token: token,
      },
    });
  }
  
  static getJoinedRoomMembers(token, roomId) {
    return simRequestHelper.post(`/rooms/${roomId}/joined_members`, {}, {
      params: {
        access_token: token,
      },
    });
  }

  static sync(token, since, isExcludeRooms = false, timeout) {
    const params = {
      access_token: token,
      filter: {
        presence: {
          not_types: ['m.presence'],
        },
        room: {
          timeline: {
            limit: 20,
          },
        },
      },
    };
    if (since) {
      params.since = since;
    }
    if (timeout) {
      params.timeout = timeout;
    }
    params.filter = JSON.stringify(params.filter);

    if (isExcludeRooms) params.exclude_rooms = true;

    return simRequestHelper.get('/sync', { params, timeout: 15000 });
  }

  static getRoomMembers(roomId) {
    return simRequestHelper.get(`/rooms/${roomId}/members`, {
      baseURL: `${baseUrl}/_synapse/admin/v1`,
      params: { access_token: asToken },
    });
  }

  static sendToDevice(token, matrixLoginId, deviceIds, eventType, txnId, content) {
    const messages = {
      [matrixLoginId]: {},
    };
    deviceIds.forEach((deviceId) => {
      messages[matrixLoginId][deviceId] = content;
    });

    return simRequestHelper.put(`/sendToDevice/${eventType}/${txnId}`, {
      messages,
    }, {
      params: {
        access_token: token,
      },
    });
  }

  static register(loginId, password) {
    return simRequestHelper.post('/register', {
      auth: {
        type: 'm.login.dummy',
      },
      username: loginId,
      password,
    });
  }

  static loginWithPassword(loginId, password) {
    return simRequestHelper.post('/login', {
      identifier: {
        type: 'm.id.user',
        user: loginId,
      },
      initial_device_display_name: 'hello',
      password,
      type: 'm.login.password',
    });
  }
}

module.exports = SimApiRequester;
